"""
    mimetypes
    ~~~~~~~~~

    Define some constants for different mimetypes
"""


CSV_MIMETYPE = 'text/csv'
FILEUPLOAD_MIMETYPE = 'multipart/form-data'
FORMURL_MIMETYPE = 'application/x-www-form-urlencoded'
JSON_MIMETYPE = 'application/json'
JSONAPI_MIMETYPE = 'application/vnd.api+json'
